Notification

Struct Notification 

#[non_exhaustive]
pub struct Notification { pub id: Uuid, pub session: Option<Uuid>, pub action: Action, pub record: Value, pub result: Value, }
A live query notification.

Fields (Non-exhaustive)

This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.
id: Uuid

The id of the LIVE query to which this notification belongs

session: Option<Uuid>

The ID of the session that sent this notification

action: Action

The CREATE / UPDATE / DELETE action which caused this notification

record: Value

The id of the document to which this notification has been made

result: Value

The resulting notification content, usually the altered record content

Implementations

Source

impl Notification

pub fn new( id: Uuid, session: Option<Uuid>, action: Action, record: Value, result: Value, ) -> Self

Construct a new notification.