pub struct SerdeWrapper<T: Serialize + DeserializeOwned + 'static>(pub T);Expand description
A wrapper struct that allows bridging between SurrealValue and types that implement Serialize and Deserialize
A note on parity with SurrealValue
Serializing and Deserializing a type does not behave the same as SurrealValue in some cases.
Notably:
- Enum variants behave differently
- The only primitives taken advantage of are String, Number, Bool, Bytes, Object, and Array
As such, it’s best to use SurrealValue directly where possible. This is intended for types where an implementation of SurrealValue isn’t available or practical
Tuple Fields
0: TTrait Implementations
Sourceimpl<T: Serialize + DeserializeOwned + 'static> SurrealValue for SerdeWrapper<T>
impl<T: Serialize + DeserializeOwned + 'static> SurrealValue for SerdeWrapper<T>